#pragma once

#include <vector>
#include <map>
#include <string>

using namespace std;

typedef unsigned char uint8;
typedef std::vector<uint8> uint8vec;

class Assembler
{
public:
	Assembler() {};
	virtual ~Assembler() {};

	bool Assemble(char* inputFilename, FILE* pInput, uint8vec* outputBuffer);

private:
	enum
	{
		kImplied = 0,
		kAbsolute,
		kImmediate,
		kIndexed,
		kNumAddressModes
	};

	struct OpcodeEntry
	{
		char* name;
		unsigned char opcodes[kNumAddressModes];
	};

	bool ParseImplied(OpcodeEntry* pEntry, char* mnemo);
	bool ParseImmediate(OpcodeEntry* pEntry, char* mnemo, char* operand);
	bool ParseAddressed(OpcodeEntry* pEntry, char* mnemo, char* operand);
	bool ParseLabel(char* label);
       bool ParseDefine(char* name, char* valueString);
       bool ParseDirective(char* directive);
	void ReportError(char* format, ...);
	char* ReadToken();
	OpcodeEntry* GetOpcodeTableEntry(const char* mnemo) const;
	int GetCurrentOutputAddress() const { return m_outputBuffer->size(); }

	static const unsigned char kInvalidOp = 0xff;
	static const int kOpcodeSize = 6;
	static const int kInputLineLength = 256;
       static const int kBlockSize = 512;

	char* m_inputFilename;
	char m_inputLine[kInputLineLength];
	int m_inputLineLength;
	int m_lineNumber;
	int m_tokenEnd;

	typedef std::map<std::string,int> LabelToAddressMap;
	LabelToAddressMap m_labels;

	struct PendingReference
	{
		int lineNumber;
		int outputAddress;

		PendingReference(int l, int a) : lineNumber(l), outputAddress(a) {}
	};

	typedef std::map<std::string,std::vector<PendingReference>> PendingReferenceList; 
	PendingReferenceList m_pendingReferences;

	uint8vec* m_outputBuffer;
};

