; assembler todo's:
; 0. .section : instead of .org? No label references across sections, and label addresses are all relative to start of the section
; 1. .byte, .ascii
; 2. Character and string literals in DB
; 3. Block comments
; 4. .if/.endif
; 5. .equ instead of #define, and .equ sym,val, or sym = value
; 6. .include
; 7. .incbin
; 8. HI() and LOW() expressions to operate on labels

; Thunks: routine per thunk (13 or 14 bytes/thunk), needs 1 byte of RAM to preserve A/X across the call, or pass args on stack
; 1. Push current bank on stack (through A)
; 2. Set bank to some hard-coded number
; 3. call routine address
; 4. Pop A and set current bank back
; 5. Return

; Thunk table with passed index (3 bytes/thunk + about 20 bytes fixed), can only pass one arg in register since X is used for thunk ID
; 1. Push current bank on stack (through A)
; 2. lda tablebase,x
; 3. pha
; 4. inx
; 5. lda tablebase,x
; 6. pha
; 7. inx
; 8. lda tablebase,x
; 9. sta ADDR_BANK
; 10. return
; etc...
; seems like the best option?

; Thunk helper routine
; caller:
; lda #HI(addr)
; pha
; lda #LOW(addr)
; pha
; lda #BANK
; call thunk_helper
; thunk_helper:
; sta BANK_ADDR
; 

; --- scratch memory byte
#define TEMP_ADDR $200

; --- device constants
#define BANK_ADDR $3BE

; --- SPI/LCD constants
#define SPI_CTRL_ADDR $3BA
#define SPI_DATA_ADDR $3BB
#define LCD_RESET_MASK $04
#define LCD_CS_MASK $10
#define LCD_DC_MASK $08
#define LCD_CS_DC_MASK $18
#define LCD_MOSI_MASK $01
#define LCD_SCK_MASK $02

; --- ST7735 LCD constants
#define ST7735_NOP $0
#define ST7735_SWRESET $01
#define ST7735_RDDID $04
#define ST7735_RDDST $09
#define ST7735_SLPIN  $10
#define ST7735_SLPOUT  $11
#define ST7735_PTLON  $12
#define ST7735_NORON  $13
#define ST7735_INVOFF $20
#define ST7735_INVON $21
#define ST7735_DISPOFF $28
#define ST7735_DISPON $29
#define ST7735_CASET $2A
#define ST7735_RASET $2B
#define ST7735_RAMWR $2C
#define ST7735_RAMRD $2E
#define ST7735_COLMOD $3A
#define ST7735_MADCTL $36
#define ST7735_FRMCTR1 $B1
#define ST7735_FRMCTR2 $B2
#define ST7735_FRMCTR3 $B3
#define ST7735_INVCTR $B4
#define ST7735_DISSET5 $B6
#define ST7735_PWCTR1 $C0
#define ST7735_PWCTR2 $C1
#define ST7735_PWCTR3 $C2
#define ST7735_PWCTR4 $C3
#define ST7735_PWCTR5 $C4
#define ST7735_VMCTR1 $C5
#define ST7735_RDID1 $DA
#define ST7735_RDID2 $DB
#define ST7735_RDID3 $DC
#define ST7735_RDID4 $DD
#define ST7735_PWCTR6 $FC
#define ST7735_GMCTRP1 $E0
#define ST7735_GMCTRN1 $E1

boot:
    lda #$01
    sta BANK_ADDR ; switch to bank 1, execution continues at next consecutive address in bank 1

    lda #0 ; dummy
    lda #0 ; dummy
    lda #0 ; dummy
    
bank_return: 
    call clear_display
    
    ldx #1
    lda #2
    
-   ; set xpos = X
    pha
    phx
    pla
    call set_xr
    pla
        
    ; set ypos = A
    phx
    pha
    plx
    pha
    call set_yr
     
    ; write to RAM
    lda #ST7735_RAMWR
    call spi_command0
         
    lda #$07
    call spi_data0
    lda #$E0
    call spi_data0    
     
    pla
    plx
    
    inx
    add #1
    cmp #12
    bne +
    lda #2       
+   cpx #161
    bne -
     
    ; halt 
-   lda #0
    jmp -
                    
; --------------------------------
; clear_display      
; --------------------------------                             
clear_display:
    ; write to RAM
    lda #ST7735_RAMWR
    call spi_command0
    lda #128
--  pha
    ldx #160
-   lda #0
    call spi_data0
    lda #0
    call spi_data0
    dex
    cpx #0
    bne -   
    pla
    sub #1
    bcs -- 
    return    
  
; --------------------------------
; set_xr: sets the start and end X range of the active display area
; inputs:
;   A: start 
;   X: end 
; --------------------------------   
set_xr:
    phx
    pha
    lda #ST7735_RASET  ; row addr set
    call spi_command0
    lda #$00
    call spi_data0
    pla       ; start row  
    call spi_data0
    lda #$00
    call spi_data0
    pla       ; end row
    call spi_data0    
    return

; --------------------------------
; set_yr: sets the start and end Y range of the active display area
; inputs:
;   A: start 
;   X: end 
; --------------------------------   
set_yr:
    phx
    pha
    lda #ST7735_CASET  ; column addr set
    call spi_command0
    lda #$00
    call spi_data0
    pla       ; start col  
    call spi_data0
    lda #$00
    call spi_data0
    pla       ; end col
    call spi_data0    
    return
    
spi_command0:
    pha
    
    ; set CS and DC low
    lda SPI_CTRL_ADDR
    nor #0
    nor #LCD_CS_DC_MASK 
    sta SPI_CTRL_ADDR
    
    jmp spi_send0
      
spi_data0:
    pha
    
    ; set CS low
    lda SPI_CTRL_ADDR
    nor #0
    nor #LCD_CS_MASK 
    ; set DC high
    nor #LCD_DC_MASK
    nor #0
    sta SPI_CTRL_ADDR
  
spi_send0:    
    pla
    sta SPI_DATA_ADDR
   
    ; wait for completion
-   lda SPI_DATA_ADDR
    bne -
        
    ; set CS high
    lda SPI_CTRL_ADDR
    nor #LCD_CS_MASK 
    nor #0
    sta SPI_CTRL_ADDR
    return
      
; ---- bank 1 ----

.org $200
    lda #0 ; dummy
    lda #0 ; dummy
    
jump1_0_target:
    call display_init
    lda #0
    sta BANK_ADDR    ; resume execution in bank 0
            
; --------------------------------
; delay: busy loop for some cycles
; inputs:
;   A: high byte of loop count
;   X: low byte of loop count
; --------------------------------   
delay_next:
    ldx #$ff 
delay:  
-   dex
    cpx #$ff ; dex does not affect carry
    bne -
    sub #1
    bcs delay_next 
    return        

; ------------------------------------------
; spi_command: send a command on the SPI bus
; inputs:
;   A: command ID
; ------------------------------------------
spi_command:
    pha
    
    ; set CS and DC low
    lda SPI_CTRL_ADDR
    nor #0
    nor #LCD_CS_DC_MASK 
    sta SPI_CTRL_ADDR
    
    jmp spi_send
    
; ------------------------------------------
; spi_command: send data on the SPI bus
; inputs:
;   A: data to send
; ------------------------------------------
spi_data:
    pha
    
    ; set CS low
    lda SPI_CTRL_ADDR
    nor #0
    nor #LCD_CS_MASK 
    ; set DC high
    nor #LCD_DC_MASK
    nor #0
    sta SPI_CTRL_ADDR
  
spi_send:    
    pla
    sta SPI_DATA_ADDR 
    
    ; wait for completion
-   lda SPI_DATA_ADDR
    bne -
    
    ; set CS high
    lda SPI_CTRL_ADDR
    nor #LCD_CS_MASK 
    nor #0
    sta SPI_CTRL_ADDR
    return
  
; -----------------------------------
; display_init: initialize ST7735 LCD
; -----------------------------------    
display_init:
    ; toggle the reset line
    lda SPI_CTRL_ADDR
    nor #0
    nor #LCD_RESET_MASK 
    sta SPI_CTRL_ADDR
    lda #$40
    ldx #0
    call delay
    lda SPI_CTRL_ADDR
    nor #LCD_RESET_MASK
    nor #0
    sta SPI_CTRL_ADDR
    lda #$40
    ldx #0
    call delay    

    ; perform software reset and sleep wake
    lda #ST7735_SWRESET
    call spi_command
    lda #$10
    ldx #0
    call delay
    lda #ST7735_SLPOUT
    call spi_command
    lda #$10
    ldx #0
    call delay
     
    lda #ST7735_FRMCTR1  ; frame rate control - normal mode
    call spi_command
    lda #$01  ; frame rate = fosc / (1 x 2 + 40) * (LINE + 2C + 2D)
    call spi_data
    lda #$2C
    call spi_data 
    lda #$2D 
    call spi_data

    lda #ST7735_FRMCTR2  ; frame rate control - idle mode
    call spi_command
    lda #$01  ; frame rate = fosc / (1 x 2 + 40) * (LINE + 2C + 2D)
    call spi_data
    lda #$2C 
    call spi_data
    lda #$2D 
    call spi_data

    lda #ST7735_FRMCTR3  ; frame rate control - partial mode
    call spi_command
    lda #$01  ; dot inversion mode
    call spi_data
    lda #$2C 
    call spi_data
    lda #$2D 
    call spi_data
    lda #$01  ; line inversion mode
    call spi_data
    lda #$2C 
    call spi_data
    lda #$2D 
    call spi_data

    lda #ST7735_INVCTR  ; display inversion control
    call spi_command
    lda #$07  ; no inversion
    call spi_data

    lda #ST7735_PWCTR1  ; power control
    call spi_command
    lda #$A2      
    call spi_data
    lda #$02  ; -4.6V
    call spi_data
    lda #$84  ; AUTO mode
    call spi_data

    lda #ST7735_PWCTR2  ; power control
    call spi_command
    lda #$C5  ; VGH25 = 2.4C VGSEL = -10 VGH = 3 * AVDD
    call spi_data

    lda #ST7735_PWCTR3  ; power control
    call spi_command
    lda #$0A  ; Opamp current small 
    call spi_data
    lda #$00  ; Boost frequency
    call spi_data

    lda #ST7735_PWCTR4  ; power control
    call spi_command
    lda #$8A  ; BCLK/2, Opamp current small & Medium low
    call spi_data
    lda #$2A     
    call spi_data

    lda #ST7735_PWCTR5  ; power control
    call spi_command
    lda #$8A    
    call spi_data
    lda #$EE     
    call spi_data

    lda #ST7735_VMCTR1  ; power control
    call spi_command
    lda #$0E  
    call spi_data

    lda #ST7735_INVOFF  ; don't invert display
    call spi_command

    lda #ST7735_MADCTL  ; memory access control (directions)
    call spi_command
    lda #$C8  ; row address/col address, bottom to top refresh
    call spi_data

    lda #ST7735_COLMOD  ; set color mode
    call spi_command
    lda #$05  ; 16-bit color
    call spi_data

    lda #ST7735_CASET  ; column addr set
    call spi_command
    lda #$00
    call spi_data
    lda #$02  ; XSTART
    call spi_data
    lda #$00
    call spi_data
    lda #$81  ; XEND
    call spi_data

    lda #ST7735_RASET  ; row addr set
    call spi_command
    lda #$00
    call spi_data
    lda #$01  ; YSTART
    call spi_data
    lda #$00
    call spi_data
    lda #$A0  ; YEND
    call spi_data

    lda #ST7735_GMCTRP1
    call spi_command
    lda #$02
    call spi_data
    lda #$1c
    call spi_data
    lda #$07
    call spi_data
    lda #$12
    call spi_data
    lda #$37
    call spi_data
    lda #$32
    call spi_data
    lda #$29
    call spi_data
    lda #$2d
    call spi_data
    lda #$29
    call spi_data
    lda #$25
    call spi_data
    lda #$2B
    call spi_data
    lda #$39
    call spi_data
    lda #$00
    call spi_data
    lda #$01
    call spi_data
    lda #$03
    call spi_data
    lda #$10
    call spi_data

    lda #ST7735_GMCTRN1
    call spi_command
    lda #$03 
    call spi_data
    lda #$1d 
    call spi_data
    lda #$07 
    call spi_data
    lda #$06 
    call spi_data
    lda #$2E 
    call spi_data
    lda #$2C 
    call spi_data
    lda #$29 
    call spi_data
    lda #$2D 
    call spi_data
    lda #$2E 
    call spi_data
    lda #$2E 
    call spi_data
    lda #$37 
    call spi_data
    lda #$3F 
    call spi_data
    lda #$00 
    call spi_data
    lda #$00 
    call spi_data
    lda #$02 
    call spi_data
    lda #$10 
    call spi_data

    lda #ST7735_DISPON
    call spi_command
    lda #$100
    ldx #0
    call delay 

    lda #ST7735_NORON  ; normal display on
    call spi_command
    lda #$10
    ldx #0
    call delay    
  
    ; write to RAM
    lda #ST7735_RAMWR
    call spi_command
      
    return