`timescale 1ns / 1ns

module testbench;

	reg clk;
	reg reset;
	reg cs;
	reg [10:0] count;
	wire [7:0] data;
	wire [9:0] address;
	wire we;
	
	ramrom ramrom0(.address(address), .cs(cs), .we(we), .data(data));
	// Fill the memory with values taken from a data file
	initial $readmemh("..\\..\\cputest16.rom", ramrom0.mem);
	
	tinycpu cpu(.clk(clk), .reset(reset), .dataBus(data), .addrBus(address), .weMem(we));

	initial begin
		reset=0;
		#225 reset=1;
	end

	initial begin
		clk=1;
		cs=0;
		count = 0;

		repeat(1000) begin
			//$monitor("%d: op=%h, intbus=%h, extbus=%h", $time, machine.contmod.op_out, machine.internal_data_bus, machine.external_data_bus);
			#50 clk = 0;
			#50 clk = 1;
			count = count + 1;
			if (cpu.state == 0 && data[7:2] == 6'h3F) begin // is CPU halted
				if (cpu.aReg == 8'hD3) begin // a has success code
					$display("SUCCESS: test passed (%d clocks)", count);
				end else begin
					$display("ERROR: test failed");
				end
				$stop;
			end
		end
		$display("ERROR: test did not terminate");
		$stop;
	end

endmodule

module ramrom ( input [9:0] address, input cs, input we, output [7:0] data );

	reg [7:0] mem[0:1023];   
	wire [7:0] dataDelayed;
	wire [9:0] addressDelayed;
	
	always @(posedge we) begin
		if (!cs)
			mem[addressDelayed] = dataDelayed;
	end

	assign data = (!cs & we) ? mem[address] : 8'bzzzzzzzz;
	
	// the delay in the RAM model causes the old values of data and address
	// to be used, where we, data, and address all change simultaneously
	assign #1 dataDelayed = data;
	assign #1 addressDelayed = address;

endmodule