module tinymcu
	(input reset,
	 inout [7:0] dataBus,
	 output [18:0] addrBus,
	 // memory interface
	 output csRam,
	 output oeRam,
	 output weRam,
	 output csRom,
	 output oeRom,
	 output weRom,	 
	 // keyboard interface
	 input keyClk, // need pull-up
	 input keyData, // need pull-up
	 // serial interface
	 input serialIn, // need pull-up, so it's in a defined state with nothing connected
	 output serialOut,
	 // lcd interface
	 output lcdReset,
	 output lcdDC,
	 output lcdCS,
	 // micro SD card interface
	 output cardCS, 
	 // SPI interface
	 output sck,
	 output mosi,
	 input miso,
	 // misc interface
	 output ledGreen,
	 output ledYellow,
	 output speaker,
	 output clkOut);
	
	wire clk26;
	internal_osc_altufm_osc_518 internal_osc(clk26, 1'b1);
	
	reg [19:0] clkdiv;
	always @(posedge clk26) begin
		clkdiv <= clkdiv + 1'b1;
	end
	
	wire clk = clk26; //clkdiv[14];
	
	assign clkOut = clk;
	
	wire weMem;
	wire [9:0] cpuAddrBus;
	assign addrBus[8:0] = cpuAddrBus[8:0];
	
	tinycpu cpu (.clk(clk), .reset(reset), .dataBus(dataBus), .addrBus(cpuAddrBus), .weMem(weMem));
	tinydevice dev (.clk(clk), .reset(reset), .dataBus(dataBus), .addrBus(cpuAddrBus), .weMem(weMem), 
						 .csRam(csRam), .oeRam(oeRam), .weRam(weRam), .csRom(csRom), .oeRom(oeRom), .weRom(weRom), 
						 .a18(addrBus[18]), .a17(addrBus[17]), .a16(addrBus[16]), .a15(addrBus[15]),
						 .addrBusHigh(addrBus[14:9]), .keyClk(keyClk), .keyData(keyData), .serialIn(serialIn), .serialOut(serialOut),
						 .lcdReset(lcdReset), .lcdDC(lcdDC), .lcdCS(lcdCS), .cardCS(cardCS), .sck(sck), .mosi(mosi), .miso(miso), 
						 .ledGreen(ledGreen), .ledYellow(ledYellow), .speaker(speaker));
	
endmodule
